<?php

abstract class Engine_Package_Manager_Operation_Abstract
{
  /**
   * @var Engine_Package_Manager
   */
  protected $_manager;

  /**
   * Package
   * @var Engine_Package_File
   */
  protected $_package;

  /**
   * Location of package file
   * @var string
   */
  protected $_packageSource;

  /**
   * Generally, APPLICATION_PATH . '/temporary/package/packages/' . $packageKey
   * @var string
   */
  protected $_sourcePath;

  /**
   * Generally, APPLICATION_PATH
   * @var string
   */
  protected $_destinationPath;



  // General
  
  public function __construct(Engine_Package_Manager $manager, Engine_Package_File $package, $options = null)
  {
    $this->_manager = $manager;

    if( is_array($options) ) {
      $this->setOptions($options);
    }

    if( null === $this->_destinationPath && defined('APPLICATION_PATH') ) {
      $this->_destinationPath = APPLICATION_PATH;
    }
    if( null === $this->_sourcePath ) {
      $this->_sourcePath = $this->_package->getBasePath();
    }
  }

  public function __sleep()
  {
    return array('_packageSource', '_sourcePath', '_destinationPath');
  }

  public function __wakeup()
  {
    $this->_package = new Engine_Package_File($this->_packageSource);
  }



  // Options

  public function getManager()
  {
    if( null === $this->_manager ) {
      throw new Engine_Package_Manager_Operation_Exception('No manager defined');
    }
    return $this->_manager;
  }

  public function setManager(Engine_Package_Manager $manager)
  {
    if( null !== $this->_manager ) {
      throw new Engine_Package_Manager_Operation_Exception('Manager already defined');
    }
    $this->_manager = $manager;
    return $this;
  }

  public function getPackage()
  {
    if( null === $this->_package ) {
      throw new Engine_Package_Manager_Operation_Exception('No package defined');
    }
    return $this->_package;
  }

  public function setOptions(array $options)
  {
    foreach( $options as $key => $value ) {
      $method = 'set' . ucfirst($key);
      if( method_exists($this, $method) ) {
        $this->$method($value);
      }
    }

    return $this;
  }

  public function setSourcePath($path)
  {
    $this->_sourcePath = $path;
    return $this;
  }

  public function getSourcePath()
  {
    if( null === $this->_sourcePath ) {
      throw new Engine_Package_Manager_Operation_Exception('No source path defined');
    }
    return $this->_sourcePath;
  }

  public function setDestinationPath($path)
  {
    $this->_destinationPath = $path;
    return $this;
  }

  public function getDestinationPath()
  {
    if( null === $this->_destinationPath ) {
      throw new Engine_Package_Manager_Operation_Exception('No destination path defined');
    }
    return $this->_destinationPath;
  }


  // Actions

  public function doDiff()
  {
    
  }

  public function doPreInstall()
  {

  }

  public function doPostInstall()
  {

  }

  abstract public function doInstall();




  // Low-level action

  public function getDiff()
  {
    return Engine_Package_Diff::compare($this->getPackage(), $this->getDestinationPath());
  }

  public function getSanity()
  {
    $battery = new Engine_Sanity(array(
      'tests' => $this->getPackage()->getTests()
    ));
    //$battery->run();
    return $battery;
  }
}