<?php
/**
 * SocialEngine
 *
 * @category   Application_Core
 * @package    Authorization
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 * @version    $Id: AdminLevelController.php 5448 2010-05-06 22:22:04Z shaun $
 * @author     John Boehr <j@webligo.com>
 */

/**
 * @category   Application_Core
 * @package    Authorization
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 */
class Authorization_AdminLevelController extends Core_Controller_Action_Admin
{
  public function indexAction()
  {
    $this->view->formFilter = $formFilter = new Authorization_Form_Admin_Level_Filter();
    $page = $this->_getParam('page',1);

    $table = $this->_helper->api()->getDbtable('levels', 'authorization');
    $select = $table->select();

    if( $this->getRequest()->isPost() && $formFilter->isValid($this->getRequest()->getPost()) )
    {
      $values = $formFilter->getValues();

      $select = $table->select()
       ->order( !empty($values['orderby']) ? $values['orderby'].' '.$values['orderby_direction'] : 'level_id DESC' );
      
      if ($values['orderby']&& $values['orderby_direction']!='ASC') $this->view->orderby = $values['orderby'];

    }

    $this->view->paginator = $paginator = Zend_Paginator::factory($select);
    $this->view->paginator = $paginator->setCurrentPageNumber( $page );
    //$this->view->formDelete = new User_Form_Admin_Manage_Delete();

  }

  public function createAction()
  {
    $this->view->form = $form = new Authorization_Form_Admin_Level_Create();

    if( $this->getRequest()->isPost() && $form->isValid($this->getRequest()->getPost()) )
    {

      $table = $this->_helper->api()->getDbtable('levels', 'authorization');
      $db = $table->getAdapter();
      $db->beginTransaction();

      try
      {
        $level = $table->createRow();
        $values = $form->getValues();
        $level->name = $values['name'];
        $level->description = $values['description'];
        $level->save();

        //@todo duplicate the settings of given parent value
        // does this go into the authorization_permission table?
        // $values['parent'];
        // select permission for the parent level
        $permissionTable = $this->_helper->api()->getDbtable('permissions', 'authorization');
        $select = $permissionTable->select()->where('level_id = ?', $values['parent']);
        $parent_permissions = $table->fetchAll($select);


        // create permissions
        foreach( $parent_permissions as $parent )
        {
          $permissions = $permissionTable->createRow();
          $permissions->setFromArray($parent->toArray());
          $permissions->level_id = $level->level_id;
          $permissions->save();
        }

        // Commit
        $db->commit();

        // Redirect
        return $this->_helper->redirector->gotoRoute(array('action' => 'index'));
        //$this->_helper->redirector->gotoRoute(array());
      }

      catch( Exception $e )
      {
        $db->rollBack();
        throw $e;
      }

    }
  }

  public function deleteAction()
  {
    $this->view->form = $form = new Authorization_Form_Admin_Level_Delete();
    $id = $this->_getParam('id', null);

    // check to make sure the level is not default
    $this->view->level = $level = Engine_Api::_()->getItem('authorization_level', $id);
    
    if( $id )
    {
      $form->level_id->setValue($id);
    }

    if( $this->getRequest()->isPost() )
    {
      $table = $this->_helper->api()->getDbtable('levels', 'authorization');
      $db = $table->getAdapter();
      $db->beginTransaction();

      try
      {
        $permissionTable = $this->_helper->api()->getDbtable('permissions', 'authorization');
        $select = $permissionTable->select()->where('level_id = ?', $id);
        $level_permissions = $permissionTable->fetchAll($select);


        // create permissions
        foreach( $level_permissions as $permission )
        {
          //this is not working
          $permission->delete();
        }
        
        $level->delete();

        $db->commit();

        // reallocate users to default level? or unassigned users are placed in them automatically?

        return $this->_helper->redirector->gotoRoute(array('action' => 'index'));
      }

      catch( Exception $e )
      {
        $db->rollBack();
        throw $e;
      }
    }
  }

  public function editAction()
  {

    $id = $this->_getParam('id', null);
    $this->view->level = $level = Engine_Api::_()->getItem('authorization_level', $id);
    $this->view->form = $form = new Authorization_Form_Admin_Level_Edit();
    
    // Posting form
    if( $this->getRequest()->isPost() )
    {
      if( $form->isValid($this->getRequest()->getPost()) )
      {
        $values = $form->getValues();
        $level->name = $values['name'];
        $level->description = $values['description'];
        $level->save();
      }

      $this->_forward('success', 'utility', 'core', array(
      'smoothboxClose' => true,
      'parentRefresh' => true,
      'format'=> 'smoothbox',
      'messages' => Zend_Registry::get('Zend_Translate')->_('Member Level Edited.')
      ));
    }

    // Initialize data
    else
    {
      $form->populate($level->toArray());
    }
  }

  public function deleteselectedAction()
  {
    // $this->view->form = $form = new Announcement_Form_Admin_Edit();
    $this->view->ids = $ids = $this->_getParam('ids', null);
    $confirm = $this->_getParam('confirm', false);
    $this->view->count = count(explode(",", $ids));

    // $announcement = Engine_Api::_()->getItem('announcement', $id);

    // Save values
    if( $this->getRequest()->isPost() && $confirm == true )
    {
      $ids_array = explode(",", $ids);

      foreach ($ids_array as $id){
        // make sure the ID is not part of the ones that cannot be deleted
        if ($id != 1 ||$id != 4||$id != 5){
          // do I check the default id and admin panel using the settings?
          $default_level = Engine_Api::_()->getApi('settings', 'core')->authorization_defaultlevel;

          // first relocate all the user with the id to the default
          $table = $this->_helper->api()->getDbtable('users', 'user');
          $userTableName = $table->info('name');
          $select = $table->select();
          $users = $table->fetchAll($select->where($userTableName.'.level_id = ?', $id));

          foreach ($users as $user){
            $user->level_id = $default_level;
            $user->save();
          }

          // delete permissions
          $permissionTable = $this->_helper->api()->getDbtable('permissions', 'authorization');
          $select = $permissionTable->select()->where('level_id = ?', $id);
          $level_permissions = $permissionTable->fetchAll($select);

          // for each level permissions delete the items
          foreach( $level_permissions as $permission )
          {
            // this is not working
            $permission->delete();
          }

          // delete
          $level = Engine_Api::_()->getItem('authorization_level', $id);
          $level->delete();
        }
      }

      //$announcement->delete();
      $this->_redirectCustom(array('route' => 'announcement_admin_general'));
    }
  }

  public function setdefaultAction(){
    $id = $this->_getParam('level_id', null);
    $table = $this->_helper->api()->getDbtable('levels', 'authorization');
    $db = $table->getAdapter();
    $db->beginTransaction();

    try
    {
      // get current default and de-flag the item
      $select = $table->select()
        ->where('flag = ?', 'default')
        ->limit(1);
      $default_level = $table->fetchRow($select);
      $default_level->flag = "";
      $default_level->save();

      // set the current item to default
      $this->view->level = $level = Engine_Api::_()->getItem('authorization_level', $id);
      $level->flag = 'default';
      $level->save();
      $db->commit();

    }

    catch( Exception $e )
    {
      $db->rollBack();
      throw $e;
    }
  }
}