<?php
/**
 * SocialEngine
 *
 * @category   Application_Core
 * @package    Core
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 * @version    $Id: Tasks.php 5304 2010-04-30 01:37:46Z john $
 */

/**
 * @category   Application_Core
 * @package    Core
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 */
class Core_Model_DbTable_Tasks extends Engine_Db_Table
{
  protected $_pendingTasks;
  
  public function hasPendingTasks()
  {
    return ( count($this->getPendingTasks()) > 0 );
  }
  
  public function getPendingTasks()
  {
    if( null === $this->_pendingTasks ) {
      $this->_pendingTasks = array();
      $now = time();
      foreach( $this->fetchAll() as $task )
      {
        if( $now > $task->executed_last + $task->timeout )
        {
          $this->_pendingTasks[] = $task;
        }
      }
    }

    return $this->_pendingTasks;
  }

  public function trigger($mode)
  {
    if( $this->hasPendingTasks() )
    {
      if( !is_string($mode) || !method_exists($this, '_trigger'.ucfirst($mode)))
      {
        throw new Core_Model_Exception('Unsupported mode: '.$mode);
      }

      $prev = ignore_user_abort(true);
      $this->{'_trigger'.ucfirst($mode)}();
      ignore_user_abort($prev);
    }

    return $this;
  }

  protected function _triggerCurl()
  {
    $multi_handle = curl_multi_init();
    $curl_handle = curl_init();
    
    // Set options
    //$url = 'http://' . $_SERVER['HTTP_HOST'] . Zend_Controller_Front::getInstance()->getBaseUrl() . '/tasks.php';
    $url = 'http://' . $_SERVER['HTTP_HOST'] . Zend_Controller_Front::getInstance()->getRouter()->assemble(array('controller' => 'utility', 'action' => 'tasks'), 'default', true) . '?notrigger=1';
    curl_setopt($curl_handle, CURLOPT_URL, $url);
    curl_setopt($curl_handle, CURLOPT_RETURNTRANSFER, true);

    // Try to handle basic htauth
    if( !empty($_SERVER['PHP_AUTH_USER']) && !empty($_SERVER['PHP_AUTH_PW']) ) {
      curl_setopt($curl_handle, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
      curl_setopt($curl_handle, CURLOPT_USERPWD, $_SERVER['PHP_AUTH_USER'] . ':' . $_SERVER['PHP_AUTH_PW']);
    }

    curl_multi_add_handle($multi_handle, $curl_handle);
    
    $active = null;
    //execute the handles
    do {
        $mrc = curl_multi_exec($multi_handle, $active);
    } while ($mrc == CURLM_CALL_MULTI_PERFORM);
    
    /*
    while ($active && $mrc == CURLM_OK) {
        if (curl_multi_select($multi_handle) != -1) {
            do {
                $mrc = curl_multi_exec($multi_handle, $active);
            } while ($mrc == CURLM_CALL_MULTI_PERFORM);
        }
    }
     * 
     */
  }
  
  public function executeTasks()
  {
    foreach( $this->getPendingTasks() as $task )
    {
      $this->executeTask($task);
    }

    return $this;
  }

  public function executeTask($task)
  {
    $task->executed_count++;
    $task->executed_last = time();
    $task->save();

    try
    {
      $plugin = Engine_Api::_()->loadClass($task->plugin);
      $plugin->executeTask();

      $task->success_count++;
      $task->save();
    }
    catch( Exception $e )
    {
      $task->failure_count++;
      $task->save();

      throw $e;
    }

    return $this;
  }
}