<?php

class Core_Plugin_Task_Mail
{
  public function executeTask()
  {
    // Should we only process if queue is enabled?
    // Note: if mail gets disabled with mail in the queue, the remaining mail
    // would not get sent
    //if( Engine_Api::_()->getApi('settings', 'core')->getSetting('core.mail.queue.enabled') ) {

    $max = Engine_Api::_()->getApi('settings', 'core')->getSetting('core_mail_count', 10);
    $count = 0;
    $break = false;
    $mailApi = Engine_Api::_()->getApi('mail', 'core');
    $mailTable = Engine_Api::_()->getDbtable('mail', 'core');
    $mailRecipientsTable = Engine_Api::_()->getDbtable('mailRecipients', 'core');
    $db = $mailTable->getAdapter();

    // Loop until no mail left or count is reached
    while( $count <= $max && !$break )
    {
      // We should run each mail in a try-catch-transaction, not all at once
      $db->beginTransaction();
      
      try
      {
        // Select a single mail item
        $mailRow = $mailTable->fetchRow($mailTable->select()->order('priority DESC')->order('mail_id ASC')->limit(1));
        if( null === $mailRow ) {
          $break = true;
          continue;
        }

        // If it's a zend type, we have to send all of them
        if( $mailRow->type == 'zend' )
        {
          $mailObject = unserialize($mailRow->body);
          if( !($mailObject instanceof Zend_Mail) ) {
            throw new Engine_Exception('mail not Zend_Mail' . get_class($mailObject) . ' ' . gettype($mailObject));
          }
          $mailApi->sendRaw($mailObject);

          $count += $mailRow->recipient_count;

          $mailRow->delete();
        }

        // We can send system type in partial batches
        else if( $mailRow->type == 'system' )
        {
          // Get recipients. If number to send > than remaining send count, only select that many
          $limit = min($max - $count, $mailRow->recipient_count);
          $recipientsRowset = $mailRecipientsTable->fetchAll($mailRecipientsTable->select()->where('mail_id = ?', $mailRow->mail_id)->limit($limit));

          // Process recipients
          $recipients = array();
          $uids = array();
          $emails = array();
          foreach( $recipientsRowset as $recipient ) {
            if( !empty($recipient->user_id) ) {
              $uids[] = $recipient->user_id;
            } else if( !empty($recipient->email) ) {
              $emails[] = $recipient->email;
            }
          }

          if( !empty($uids) ) {
            $recipientObjects = Engine_Api::_()->getItemTable('user')->find($uids);
            foreach( $recipientObjects as $recipientObject ) {
              $recipients[] = $recipientObject;
            }
          }

          if( !empty($emails) ) {
            $recipients += $emails;
          }


          // Send
          $params = unserialize($mailRow->body);
          $mailApi->sendSystemRaw($recipients, $params['type'], $params['params']);
          
          // Decrement/delete recipients and/or remove row
          // Amount sent was equal to remaining count
          if( $limit >= $mailRow->recipient_count ) {
            $mailRecipientsTable->delete(array(
              'mail_id = ?' => $mailRow->mail_id,
            ));

            $mailRow->delete();
          }
          // Amount sent was less than remaining count
          else
          {
            $mailRecipientsTable->delete(array(
              'mail_id = ?' => $mailRow->mail_id,
              'user_id IN(?)' => $uids,
            ));
            
            $mailRow->recipient_count -= $limit;
            $mailRow->save();
          }
        }

        $db->commit();
      }

      catch( Exception $e )
      {
        $db->rollBack();
        // Re throw?
        throw $e;
      }
    }

    //}
  }
}