<?php
/**
 * SocialEngine
 *
 * @category   Application_Core
 * @package    Core
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 * @author     Jung
 */

/**
 * @category   Application_Core
 * @package    Core
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 */
class Core_AdminAdsController extends Core_Controller_Action_Admin
{

  // Ad Campaign actions
  public function indexAction(){
    $this->view->viewer = $this->_helper->api()->user()->getViewer();

    $paginator = Engine_Api::_()->getApi('Adcampaign', 'core')->getPaginator();
    $this->view->paginator = $paginator->setCurrentPageNumber( 1 );
  }

  public function createAction(){
    $this->view->navigation = $this->_helper->api()
      ->getApi('menus', 'core')
      ->getNavigation('core_admin_main');

    $this->view->form = $form = new Core_Form_Admin_Ads_Create();
   
    if( $this->getRequest()->isPost() && $form->isValid($this->getRequest()->getPost()) )
    {
      $params = $form->getValues();
      $campaign = $this->_helper->api()->getDbtable('Adcampaigns', 'core')->createRow();
      $campaign->setFromArray($params);
      $campaign->network = Zend_Json_Encoder::encode($params['ad_networks']);
      $campaign->level = Zend_Json_Encoder::encode($params['ad_levels']);
      $campaign->save();

      // redirect to manage page for now
      $this->_helper->redirector->gotoRoute(array('action' => 'index'));
    }

  }

  public function editAction(){
    $id = $this->_getParam('id', null);
    $this->view->navigation = $navigation = Engine_Api::_()->getApi('menus', 'core')
      ->getNavigation('adcampaign_admin_main', array('params'=>array('id'=>$id)), 'adcampaign_admin_main_edit');

    $this->view->form = $form = new Core_Form_Admin_Ads_Edit();
    $this->view->campaign = $campaign = Engine_Api::_()->getItem('core_adcampaign', $id);

    // Save values
    if( $this->getRequest()->isPost() && $form->isValid($this->getRequest()->getPost()) )
    {
      $params = $form->getValues();
      //$params['user_id'] = $this->_helper->api()->user()->getViewer()->getIdentity();
      //$announcement = $this->_helper->api()->getDbtable('announcements', 'announcement')->createRow();
      $campaign->setFromArray($params);

      $selected_levels = $this->_getParam('ad_levels');
      $selected_networks = $this->_getParam('ad_networks');

      $campaign->network = Zend_Json_Encoder::encode($selected_networks);
      $campaign->level = Zend_Json_Encoder::encode($selected_levels);

      $campaign->save();
      //$this->_helper->redirector->gotoRoute(array('action' => 'index'));
      //$form->addMessage('Changes Saved!');
    }

    // Initialize values
    else
    {
      $form->populate($campaign->toArray());

      // set the networks & level values
      //$networks = Engine_Api::_()->network()->getNetwork(NULL, NULL);
      $networks = Engine_Api::_()->getDbtable('networks', 'network')->fetchAll();
      if($networks){
        $form->getElement('ad_networks')->setValue(Zend_Json_Decoder::decode($campaign->network));
      }
      $form->getElement('ad_levels')->setValue(Zend_Json_Decoder::decode($campaign->level));

      /*
       * $this->view->selected_levels = $selected_levels = Zend_Json_Decoder::decode($campaign->level);
       * $this->view->selected_networks = $selected_networks = Zend_Json_Decoder::decode($campaign->network);
       */
    }
  }

  public function deleteAction(){

    // delete associated advertisements

    // remove the ad campaing
    $this->view->form = $form = new Core_Form_Admin_Ads_Delete();
    $id = $this->_getParam('id', null);

    if( $id )
    {
      $form->adcampaign_id->setValue($id);
    }

    if( $this->getRequest()->isPost() )
    {
      $table = $this->_helper->api()->getDbtable('Adcampaigns', 'core');
      $db = $table->getAdapter();
      $db->beginTransaction();

      try
      {
        $campaign = Engine_Api::_()->getItem('core_adcampaign', $id);
        $ads = $campaign->getAds();
        foreach($ads as $ad){
          Engine_Api::_()->getApi('Ad', 'core')->deleteAd($ad);
        }
        $campaign->delete();
        
        $db->commit();

        $this->_forward('success', 'utility', 'core', array(
        'smoothboxClose' => true,
        'parentRefresh' => true,
        'format'=> 'smoothbox',
        'messages' => array(Zend_Registry::get('Zend_Translate')->_("Ad Campaign Deleted"))
        ));
      }

      catch( Exception $e )
      {
        $db->rollBack();
        throw $e;
      }
    }
  }
 
  public function statusAction(){
    $id = $this->_getParam('adcampaign_id', null);
    $table = $this->_helper->api()->getDbtable('Adcampaigns', 'core');
    $db = $table->getAdapter();
    $db->beginTransaction();

    try
    {
      $campaign = Engine_Api::_()->getItem('core_adcampaign', $id);

      // make status paused or started depending on current setting
      if($campaign->status){
        $campaign->status =0;
      }
      else $campaign->status = 1;
      $campaign->save();

      $db->commit();

    }

    catch( Exception $e )
    {
      $db->rollBack();
      throw $e;
    }
  }




  public function manageadsAction(){
    $this->view->campaign_id = $id = $this->_getParam('id', null);
    $this->view->navigation = $navigation = Engine_Api::_()->getApi('menus', 'core')
      ->getNavigation('adcampaign_admin_main', array('params'=>array('id'=>$id)), 'adcampaign_admin_main_manageads');
    $this->view->campaign = $campaign = Engine_Api::_()->getItem('core_adcampaign', $id);

    $campaign = Engine_Api::_()->getItem('core_adcampaign', $id);

    $this->view->ads = $campaign->getAds();
    
  }

  // Ad action actions
  public function createadAction(){
   // die('here');
    if( isset($_GET['ul']) || isset($_FILES['Filedata']) ) return $this->_forward('upload-photo', null, null, array('format' => 'json'));


    $id = $this->_getParam('id', null);
    $this->view->campaign = $campaign = Engine_Api::_()->getItem('core_adcampaign', $id);

    $this->view->navigation = $navigation = Engine_Api::_()->getApi('menus', 'core')
      ->getNavigation('adcampaign_admin_main', array('params'=>array('id'=>$id)), 'adcampaign_admin_main_manageads');

    $this->view->form = $form = new Core_Form_Admin_Ads_Ad();

    if( $this->getRequest()->isPost() && $form->isValid($this->getRequest()->getPost()) )
    {
      $params = $form->getValues();
      $ad = $this->_helper->api()->getDbtable('Ads', 'core')->createRow();
      $ad->setFromArray($params);
      $ad->ad_campaign = $id;

      $ad->save();

      // redirect to manage page for now
      $this->_helper->redirector->gotoRoute(array('action' => 'manageads', 'id'=>$id));
    }
  }

  public function editadAction(){
    $this->view->form = $form = new Core_Form_Admin_Ads_Adedit();
    $id = $this->_getParam('id', null);
    $campaign = Engine_Api::_()->getItem('core_ad', $id);

    // Save values
    if( $this->getRequest()->isPost() && $form->isValid($this->getRequest()->getPost()) )
    {
      $params = $form->getValues();
      //$params['user_id'] = $this->_helper->api()->user()->getViewer()->getIdentity();
      //$announcement = $this->_helper->api()->getDbtable('announcements', 'announcement')->createRow();
      $campaign->setFromArray($params);
      $campaign->save();

      $this->_forward('success', 'utility', 'core', array(
      'smoothboxClose' => true,
      'parentRefresh' => true,
      'format'=> 'smoothbox',
      'messages' => array(Zend_Registry::get('Zend_Translate')->_("Advertisement Edited."))
      ));
    }

    // Initialize values
    else
    {
      $form->populate($campaign->toArray());
    }
  }

  public function deleteadAction(){
    $this->view->form = $form = new Core_Form_Admin_Ads_Addelete();
    $id = $this->_getParam('id', null);

    if( $id )
    {
      $form->ad_id->setValue($id);
    }

    if( $this->getRequest()->isPost() )
    {
      $table = $this->_helper->api()->getDbtable('Ads', 'core');
      $db = $table->getAdapter();
      $db->beginTransaction();

      try
      {
        $ad = Engine_Api::_()->getItem('core_ad', $id);
        Engine_Api::_()->getApi('Ad', 'core')->deleteAd($ad);
        $db->commit();

        $this->_forward('success', 'utility', 'core', array(
        'smoothboxClose' => true,
        'parentRefresh' => true,
        'format'=> 'smoothbox',
        'messages' => array(Zend_Registry::get('Zend_Translate')->_("Advertisement Deleted."))
        ));
      }

      catch( Exception $e )
      {
        $db->rollBack();
        throw $e;
      }
    }
  }

  // code to handle upload stuff

  public function uploadPhotoAction()
  {
    if( !$this->_helper->requireUser()->checkRequire() )
    {
      $this->view->status = false;
      $this->view->error = Zend_Registry::get('Zend_Translate')->_("Max file size limit exceeded (probably).");
      return;
    }

    if( !$this->getRequest()->isPost() )
    {
      $this->view->status = false;
      $this->view->error = Zend_Registry::get('Zend_Translate')->_("Invalid request method");
      return;
    }

    $values = $this->getRequest()->getPost();
    if( empty($values['Filename']) )
    {
      $this->view->status = false;
      $this->view->error = Zend_Registry::get('Zend_Translate')->_("No file");
      return;
    }

    if( !isset($_FILES['Filedata']) || !is_uploaded_file($_FILES['Filedata']['tmp_name']) )
    {
      $this->view->status = false;
      $this->view->error = Zend_Registry::get('Zend_Translate')->_("Invalid Upload");
      return;
    }
    $table = $this->_helper->api()->getDbtable('Adphotos', 'core');
    $db = $table->getAdapter();
    $db->beginTransaction();

    try
    {
      $viewer = Engine_Api::_()->user()->getViewer();

      $params = array(
        'owner_type' => 'user',
        'owner_id' => $viewer->getIdentity()
      );
      $photo_id = Engine_Api::_()->getApi('Ad', 'core')->createPhoto($params, $_FILES['Filedata'])->adphoto_id;

      $this->view->status = true;
      $this->view->name = $_FILES['Filedata']['name'];
      $this->view->photo_id = $photo_id;
      $this->view->photo_url = "<a href='' target='_blank'><img src='".Engine_Api::_()->getItem('core_adphoto', $photo_id)->getPhotoUrl()."'/></a>";


      $db->commit();
    }

    catch( Exception $e )
    {
      $db->rollBack();
      $this->view->status = false;
      $this->view->error = 'An error occurred.'.$e;
      // throw $e;
      return;
    }
  }

  public function previewAction(){
    $id = $this->_getParam('id', null);
    $campaign = Engine_Api::_()->getItem('core_ad', $id);
    $photo_id = $campaign->photo_id;

    $preview = $campaign->html_code;

    $this->view->preview = $preview;

  }

  public function removephotoAction()
  {
    $viewer = Engine_Api::_()->user()->getViewer();
    
    $photo_id= (int) $this->_getParam('photo_id');
    $photo = Engine_Api::_()->getItem('core_adphoto', $photo_id);

    $db = $photo->getTable()->getAdapter();
    $db->beginTransaction();

    try
    {
      $photo->delete();
      // @todo need to delete it out of storage system
      $db->commit();
    }

    catch( Exception $e )
    {
      $db->rollBack();
      throw $e;
    }
  }


  // audience stuff
  // deprecated: merged in to create and edit action
  public function audienceAction(){
    $id = $this->_getParam('id', null);
    $this->view->navigation = $navigation = Engine_Api::_()->getApi('menus', 'core')
      ->getNavigation('adcampaign_admin_main', array('params'=>array('id'=>$id)), 'adcampaign_admin_main_audience');

    $campaign = Engine_Api::_()->getItem('core_adcampaign', $id);

    // selected stuff
    $this->view->selected_levels = $selected_levels = $this->_getParam('ad_levels');
    $this->view->selected_networks = $selected_networks = $this->_getParam('ad_networks');
    if ($selected_levels||$selected_networks){
     $campaign->network = Zend_Json_Encoder::encode($selected_networks);
     $campaign->level = Zend_Json_Encoder::encode($selected_levels);
     $campaign->save();
    }
    else{
      $this->view->selected_levels = $selected_levels = Zend_Json_Decoder::decode($campaign->level);
      $this->view->selected_networks = $selected_networks = Zend_Json_Decoder::decode($campaign->network);
    }

    // prepare levels
    $table = $this->_helper->api()->getDbtable('levels', 'authorization');
    $this->view->levels = $table->fetchAll();

    // prepare networks
    $this->view->networks = Engine_Api::_()->network()->getNetwork(NULL, NULL);
  }
}