<?php

class Core_AdminPluginsController extends Core_Controller_Action_Admin
{
  public function indexAction()
  {
    $moduleTable = Engine_Api::_()->getDbtable('modules', 'core');
    $moduleSelect = $moduleTable->select()
      ->where('type = ?', 'extra')
      ;

    $this->view->modules = $modules = $moduleTable->fetchAll($moduleSelect);

    $new_modules = $this->view->new_modules = array();
    foreach (glob(APPLICATION_PATH . '/application/modules/*', GLOB_ONLYDIR) as $dir) {
      if (!is_dir($dir))
        continue;

      $installed = false;
      foreach ($moduleTable->fetchAll() as $module)
        if ($module->name == Engine_Api::deflect(basename($dir)))
          $installed = true;

      // temporary to show install
      if (preg_match('/video$/', $dir))
        $installed = true;

      if (!$installed) {
        $meta = include "$dir/settings/manifest.php";
        if (isset($meta['_meta']['name']))
          $new_modules[] = (object) array(
            'title'   => ucwords(str_replace('-', ' ', $meta['_meta']['name'])),
            'name'    => $meta['_meta']['name'],
            'version' => $meta['_meta']['version'],
            'description' => '',
            );
      }
    }
    $this->view->new_modules = $new_modules;
  }

  public function installAction()
  {
    $sql = APPLICATION_PATH . '/application/modules/' . Engine_Api::inflect($this->_getParam('name')) . '/settings/my.sql';
    if (!file_exists($sql)) {
      $this->_forward('success', 'utility', 'core', array(
        'messages' => array('Unable to find module-required file in /settings/my.sql'),
        'parentRefresh' => 5000,
      ));
    }

    // protect already-installed modules
    if ( Engine_Api::_()->getDbtable('modules', 'core')->getModule($this->_getParam('name')) ) {
      $this->_forward('success', 'utility', 'core', array(
        'messages' => array('This module has already been installed'),
        'parentRefresh' => 2000,
      ));
      return;
    }

    

    $db = Engine_Db_Table::getDefaultAdapter();
    $db->beginTransaction();
    try {
      foreach (Engine_Package_Utilities::sqlSplit(file_get_contents($sql)) as $query) {
        $query = trim($query);
        if (!empty($query))
          $db->query($query);
      }
      $db->commit();
      $this->_forward('success', 'utility', 'core', array(
        'messages' => array('Plugin successfully installed.'),
        'parentRefresh' => 2000,
      ));
    } catch (Exception $e) {
      $db->rollBack();
      $this->_forward('success', 'utility', 'core', array(
        'messages' => array('Unable to install all module-required database entries.', $e->getMessage()),
        'parentRefresh' => 5000,
      ));
    }
  }

  public function disableAction()
  {
    $module = $this->_getParam('name');
    $moduleRow = Engine_Api::_()->getDbtable('modules', 'core')->find($module)->current();
    if( !$module || !$moduleRow || $moduleRow == 'core' ) {
      $this->view->status = false;
      return;
    }

    $moduleRow->enabled = false;
    $moduleRow->save();

    $this->view->status = true;

    $this->_redirect('admin/plugins');
  }

  public function enableAction()
  {
    $module = $this->_getParam('name');
    $moduleRow = Engine_Api::_()->getDbtable('modules', 'core')->find($module)->current();
    if( !$module || !$moduleRow || $moduleRow == 'core' ) {
      $this->view->status = false;
      return;
    }

    $moduleRow->enabled = true;
    $moduleRow->save();

    $this->view->status = true;

    $this->_redirect('admin/plugins');
  }
}