<?php
/**
 * SocialEngine
 *
 * @category   Application_Core
 * @package    Fields
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 * @version    $Id: Search.php 5606 2010-05-11 06:29:30Z john $
 * @author     John Boehr <j@webligo.com>
 */

/**
 * @category   Application_Core
 * @package    Fields
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 * @author     John Boehr <j@webligo.com>
 */
class Fields_Form_Search extends Engine_Form
{
  protected $_fieldType;

  protected $_topLevelId;

  protected $_topLevelValue;

  public function __construct($options)
  {
    Fields_Form_Standard::enableForm($this);
    parent::__construct($options);
  }

  public function setType($type)
  {
    return $this->setFieldType($type);
  }

  public function setFieldType($type)
  {
    $this->_fieldType = $type;
    return $this;
  }

  public function setTopLevelId($id)
  {
    $this->_topLevelId = $id;
    return $this;
  }

  public function getTopLevelId()
  {
    return $this->_topLevelId;
  }

  public function setTopLevelValue($val)
  {
    $this->_topLevelValue = $val;
    return $this;
  }

  public function getTopLevelValue()
  {
    return $this->_topLevelValue;
  }

  public function init()
  {
    $this->addDecorators(array(
      'FormElements',
      array(array('li' => 'HtmlTag'), array('tag' => 'ul')),
      array('HtmlTag', array('tag' => 'div', 'class' => 'field_search_criteria'))
    ));

    // special helper code to get the display name element
    $this->generate();
  }

  public function generate()
  {
    // get the search structure
    $structure = Engine_Api::_()->getApi('core', 'fields')->getFieldsStructureSearch($this->_fieldType);

    // Start firing away
    $index = 0;
    foreach( $structure as $map )
    {
      $field = $map->getChild();
      $index++;

      // Ignore fields not searchable (even though getFieldsStructureSearch should have skipped it already)
      if( !$field->search )
      {
        continue;
      }

      // Get search key
      $key = null;
      if( !empty( $field->alias) ) {
        $key =  $field->alias;
      } else {
        $key = sprintf('field_%d', $field->field_id);
      }

      // Get params
      $params = $field->getElementParams($this->_fieldType, array('required' => false));

      if( !@is_array($params['options']['attribs']) ) {
        $params['options']['attribs'] = array();
      }

      // Change decorators
      $params['options']['decorators'] = array(
        'ViewHelper',
        array('Label', array('tag' => 'span')),
        array('HtmlTag', array('tag' => 'li'))
      );
      $params['options']['order'] = $index;

      // Hack birthdate->age
      if( $field->type == 'birthdate' ) {
        $params['type'] = 'Select';
        $params['options']['label'] = 'Age';
        $multiOptions = array('' => ' ');
        for( $i = 13; $i <= 100; $i++ ) {
          $multiOptions[$i] = $i;
        }
        $params['options']['multiOptions'] = $multiOptions;
      }
      
      // Hacks
      switch( $field->type ) {
        // Ignored fields (these are hard-coded)
        case 'profile_type':
        case 'first_name':
        case 'last_name':
          continue;
          break;
        
        // Ranges
        case 'date':
        case 'birthdate':
        case 'int':
        case 'integer':
        case 'float':
          // Use subform
          $subform = new Zend_Form_SubForm(array(
            'description' => $params['options']['label'],
            'order' => $params['options']['order'],
            'decorators' => array(
              'FormElements',
              array('Description', array('placement' => 'PREPEND', 'tag' => 'span')),
              array('HtmlTag', array('tag' => 'li'))
            )
          ));
          unset($params['options']['label']);
          unset($params['options']['order']);
          $params['options']['decorators'] = array('ViewHelper');
          $subform->addElement($params['type'], 'min', $params['options']);
          $subform->addElement($params['type'], 'max', $params['options']);
          $this->addSubForm($subform, $key);

          break;

        // Normals
        default:

          // Use element
          $params['options']['onkeypress']='return disableEnterKey(event)';
          $this->addElement($params['type'], $key, $params['options']);
          
          break;
      }

      $element = $this->getElement($key);
    }
    
    // Add submit button
    $this->addElement('Button', 'done', array(
      'label' => 'Search',
      'order' => 2000000,
      'onclick' => 'javascript:searchMembers()'
    ));
  }
}