<?php
/**
 * SocialEngine
 *
 * @category   Application_Extensions
 * @package    Forum
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 * @version    $Id: Forum.php 4677 2010-03-29 18:41:52Z john $
 * @author     John Boehr <j@webligo.com>
 */

/**
 * @category   Application_Extensions
 * @package    Forum
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 */
class Forum_Model_Forum extends Core_Model_Item_Collectible
{
  protected $_children_types = array('forum_topic');

  protected $_parent_type = 'forum_category';

  protected $_owner_type = 'forum_category';

  protected $_collection_type = 'forum_category';

  protected $_collection_column_name = 'category_id';

  //We use membership system to manage moderators
  public function membership()
  {
    return new Engine_ProxyObject($this, $this->api()->getDbtable('membership', 'forum'));
  }  

  public function getCollection()
  {
    return Engine_Api::_()->getItem($this->_collection_type, $this->category_id);

  }

  public function getHref($params = array())
  {
    $params = array_merge(array(
      'route' => 'forum_forum',
      'reset' => true,
      'action' => 'view',
      'forum_id' => $this->getIdentity(),
    ), $params);
    $route = $params['route'];
    $reset = $params['reset'];
    unset($params['route']);
    unset($params['reset']);
    return Zend_Controller_Front::getInstance()->getRouter()
      ->assemble($params, $route, $reset);
  }


  public function getLastUpdatedTopic()
  {
    return $this->getChildren('forum_topic', array('limit'=>1, 'order'=>'modified_date DESC'))->current();
  }

  // Hooks
  
  protected function _insert()
  {
    if( empty($this->category_id) ) {
      throw new Forum_Model_Exception('Cannot have a forum without a category');
    }

    // Increment parent forum count
    $category = $this->getParent();
    $category->forum_count++;
    $category->save();

    parent::_insert();
  }

  protected function _update()
  {
    if( empty($this->category_id) ) {
      throw new Forum_Model_Exception('Cannot have a forum without a category');
    }

    parent::_update();
  }

  protected function _delete()
  {
    // Decrement parent forum count
    $category = $this->getParent();
    $category->forum_count--;
    $category->save();

    // Delete all child topics
    $table = Engine_Api::_()->getItemTable('forum_topic');
    $select = $table->select()
      ->where('forum_id = ?', $this->getIdentity())
      ;
    foreach( $table->fetchAll($select) as $topic )
    {
      $topic->delete();
    }

    
    parent::_delete();
  }

  public function isModerator($user)
  {
    $list = $this->getModeratorList();
    return $list->has($user);
  }

  public function getModeratorList()
  {
    $table = Engine_Api::_()->getItemTable('forum_list');
    $select = $table->select()
      ->where('owner_id = ?', $this->getIdentity())
      ->limit(1);

    $list = $table->fetchRow($select);

    if( null === $list ) {
      $list = $table->createRow();
      $list->setFromArray(array(
        'owner_id' => $this->getIdentity(),
      ));
      $list->save();
    }

    return $list;
  }



}