<?php
/**
 * SocialEngine
 *
 * @category   Application_Extensions
 * @package    Music
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 * @version    $Id: Global.php 4379 2010-03-16 03:00:07Z jung $
 * @author     Stephen
 */

/**
 * @category   Application_Extensions
 * @package    Music
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 */
class Music_Form_Admin_Level extends Engine_Form
{
  protected $_roles = array(
        'everyone'       => 'Everyone',
        'owner_network' => 'Friends and Networks',
        'owner_member_member'  => 'Friends of Friends',
        'owner_member'         => 'Friends Only',
        'owner'          => 'Just Me'
      );

  public function init()
  {
    $this
      ->setTitle('Member Level Settings')
      ->setDescription('MUSIC_FORM_ADMIN_LEVEL_DESCRIPTION');

    $settings = Engine_Api::_()->getApi('settings', 'core');

    /*
    // Get rid of decorators for this form
    $this->clearDecorators();
    $this->addDecorator('FormElements')
         ->addDecorator('Form')
         ->addDecorator('FormTitle', array('tag' => 'h3', 'placement' => 'PREPEND'))
         ->setAction(Zend_Controller_Front::getInstance()->getRouter()->assemble(array()))
         ;
*/
    
    $levels = array();
    $table  = Engine_Api::_()->getDbtable('levels', 'authorization');
    foreach ($table->fetchAll($table->select()) as $row)
      $levels[$row['level_id']] = $row['name'];
    $this->addElement('Select', 'level_id', array(
      'label' => 'Member Level',
      'multiOptions' => $levels,
    ));

    $this->addElement('Radio', 'view', array(
      'label' => 'Allow Viewing of Playlists?',
      'description' => 'MUSIC_FORM_ADMIN_LEVEL_VIEW_DESCRIPTION',
      'multiOptions' => array(
        0 => 'No, do not allow playlists to be viewed.',
        1 => 'Yes, allow viewing of playlists.',
        2 => 'Yes, allow viewing of all playlists, even private ones.'
      ),
      'value' => 1,
    ));

    $this->addElement('Radio', 'edit', array(
      'label' => 'Allow Editing of Playlists?',
      'description' => 'MUSIC_FORM_ADMIN_LEVEL_EDIT_DESCRIPTION',
      'multiOptions' => array(
        0 => 'No, do not allow members to edit their playlists.',
        1 => 'Yes, allow members to edit their own playlists.',
        2 => 'Yes, allow members to edit all playlists.'
      ),
      'value' => 1,
    ));

    $this->addElement('Radio', 'delete', array(
      'label' => 'Allow Deletion of Playlists?',
      'description' => 'MUSIC_FORM_ADMIN_LEVEL_DELETE_DESCRIPTION',
      'multiOptions' => array(
        0 => 'No, do not allow members to delete their playlists.',
        1 => 'Yes, allow members to delete their own playlists.',
        2 => 'Yes, allow members to delete all playlists.'
      ),
      'value' => 1,
    ));

    $this->addElement('Radio', 'create', array(
      'label' => 'Allow Music?',
      'description' => 'Do you want to allow users to upload music to their profile?',
      'multiOptions' => array(
        1 => 'Yes, allow this member level to create playlists',
        0 => 'No, do not allow this member level to create playlists',
      ),
      'value' => 1,
    ));

    // PRIVACY ELEMENTS
    $this->addElement('MultiCheckbox', 'auth_view', array(
      'label' => 'Playlist Privacy',
      'description' => 'MUSIC_FORM_ADMIN_LEVEL_AUTHVIEW_DESCRIPTION',
      'multiOptions' => $this->_roles,
      'value' => array_keys($this->_roles)
    ));

    $this->addElement('MultiCheckbox', 'auth_comment', array(
      'label' => 'Playlist Comment Options',
      'description' => 'MUSIC_FORM_ADMIN_LEVEL_AUTHCOMMENT_DESCRIPTION',
      'multiOptions' => $this->_roles,
      'value' => array_keys($this->_roles)
    ));


    $this->addElement('Text', 'max_songs', array(
      'label' => 'Maximum Allowed Songs',
      'description' => 'MUSIC_FORM_ADMIN_LEVEL_MAXSONGS_DESCRIPTION',
      'value' => $settings->getSetting("music.maxSongsDefault", 30),
    ));

    $this->addElement('Text', 'max_filesize', array(
      'label' => 'Maximum Allowed Filesize',
      'description' => 'MUSIC_FORM_ADMIN_LEVEL_MAXFILESIZE_DESCRIPTION',
      'value' => $settings->getSetting("music.maxFilesizeDefault", 10000),
    ));

    $this->addElement('Text', 'max_storage', array(
      'label' => 'Maximum Allowed Storage',
      'description' => 'How much storage space in KB should each user have to store their files?',
      'value' => $settings->getSetting("music.maxStorageDefault", 100000),
    ));

    // Add submit button
    $this->addElement('Button', 'submit', array(
      'label' => 'Save Changes',
      'type' => 'submit',
      'ignore' => true
    ));
  }

  /* legacy code
  public function populate($settings)
  {
    foreach ($settings as $key => $value) {
      if ($this->getElement($key)) {
        // only pass keys, but only if it's an associative array
        if (is_array($value)) {
            if (is_string( array_shift(array_keys($value)) ))
              $value = array_keys($value);
        // decode JSON arrays
        } elseif ('[' == substr($value, 0, 1)) {
            $value = Zend_Json_Decoder::decode($value);
        // decode JSON objects
        } elseif ('{' == substr($value, 0, 1)) {
            $value = Zend_Json_Decoder::decode($value);
            if (is_array($value))
                $value = array_keys($value);
        // moderators have 2 set; convert to 1 for display
        } elseif (2 == $value && $key != 'level_id') {
            $value = 1;
        }
        $this->getElement($key)->setValue($value);
      }
    }
  }
   *
   * +
   */
}