<?php
/**
 * SocialEngine
 *
 * @category   Application_Extensions
 * @package    Music
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 * @version    $Id: Create.php 5254 2010-04-28 01:02:48Z jung $
 * @author     Steve
 */

/**
 * @category   Application_Extensions
 * @package    Music
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 */
class Music_Form_Create extends Engine_Form
{
  protected $_playlist;
  protected $_roles = array(
        'everyone'             => 'Everyone',
        'owner_network' => 'Friends and Networks',
        'owner_member_member'  => 'Friends of Friends',
        'owner_member'         => 'Friends Only',
        'owner'                => 'Just Me'
      );

  public function init()
  {
    // Init form
    $this
      ->setTitle('Add New Songs')
      ->setDescription('Choose music from your computer to add to this playlist.')
      ->setAttrib('id',      'form-upload-music')
      ->setAttrib('name',    'playlist_create')
      ->setAttrib('enctype', 'multipart/form-data')
      ->setAction(Zend_Controller_Front::getInstance()->getRouter()->assemble(array()))
      ;

    // Init name
    $this->addElement('Text', 'title', array(
      'label' => 'Playlist Name',
      'maxlength' => '63',
      'filters' => array(
        //new Engine_Filter_HtmlSpecialChars(),
        new Engine_Filter_Censor(),
        new Engine_Filter_StringLength(array('max' => '63')),
      )
    ));

    // Init descriptions
    $this->addElement('Textarea', 'description', array(
      'label' => 'Playlist Description',
      'maxlength' => '300',
      'filters' => array(
        'StripTags',
        //new Engine_Filter_HtmlSpecialChars(),
        new Engine_Filter_Censor(),
        new Engine_Filter_StringLength(array('max' => '300')),
        new Engine_Filter_EnableLinks(),
      ),
    ));

    // Init search checkbox
    $this->addElement('Checkbox', 'search', array(
      'label' => "Show this playlist in search results",
      'value' => 1,
      'checked' => true,
    ));


    // AUTHORIZATIONS
    $user_level   = Engine_Api::_()->user()->getViewer()->level_id;
    $allowed_view = @Engine_Api::_()->getApi('core', 'authorization')->getPermission($user_level, 'music_playlist', 'auth_view');
    if (!empty($allowed_view) && strlen($allowed_view) > 2) {
      $allowed_view = Zend_Json_Decoder::decode($allowed_view);
      $viewPerms    = array();
      foreach( $allowed_view as $allowed ){
        $viewPerms[$allowed] = $this->_roles[$allowed];
      }
      $this->addElement('Select', 'auth_view', array(
        'label'        => 'Privacy',
        'description'  => 'Who may see this playlist?',
        'multiOptions' => $viewPerms,
        'value'        => array_shift(array_keys($viewPerms)),
      ));
      $this->auth_view->getDecorator('Description')->setOption('placement', 'append');
    }

    $allowed_comment = @Engine_Api::_()->authorization()->getPermission($user_level, 'music_playlist', 'auth_comment');
    if (!empty($allowed_comment) && strlen($allowed_comment) > 2){
      $allowed_comment = Zend_Json_Decoder::decode($allowed_comment);
      $commentPerms    = array();
      foreach( $allowed_comment as $allowed ){
        $commentPerms[$allowed] = $this->_roles[$allowed];
      }
      $this->addElement('Select', 'auth_comment', array(
        'label'        => 'Comment Privacy',
        'description'  => 'Who may post comments on this playlist?',
        'multiOptions' => $commentPerms,
        'value'        => array_shift(array_keys($commentPerms))
      ));
      $this->auth_comment->getDecorator('Description')->setOption('placement', 'append');
    }


    // Init playlist art
    $this->addElement('File', 'art', array(
      'label' => 'Playlist Artwork',
    ));

    // Init file uploader
    $fancyUpload = new Engine_Form_Element_FancyUpload('file');
    $fancyUpload->clearDecorators()
                ->addDecorator('FormFancyUpload')
                ->addDecorator('viewScript', array(
                  'viewScript' => '_FancyUpload.tpl',
                  'placement'  => '',
                  ));
    Engine_Form::addDefaultDecorators($fancyUpload);
    $this->addElement($fancyUpload);

    // Init hidden file IDs
    $this->addElement('Hidden', 'fancyuploadfileids');


    // Init submit
    $this->addElement('Button', 'submit', array(
      'label' => 'Save Music to Playlist',
      'type'  => 'submit',
    ));
  }

  public function clearUploads()
  {
    $this->getElement('fancyuploadfileids')->setValue('');
  }

  public function saveValues()
  {
    $playlist = null;
    $values   = $this->getValues();
    $translate= Zend_Registry::get('Zend_Translate');
    
    if(!empty($values['playlist_id']))
      $playlist = Engine_Api::_()->getItem('music_playlist', $values['playlist_id']);
    else {
      $playlist = $this->_playlist = Engine_Api::_()->getDbtable('playlists', 'music')->createRow();
      $playlist->title = trim($values['title']);
      if (empty($playlist->title))
        $playlist->title = $translate->_('_MUSIC_UNTITLED_PLAYLIST');
      
      $playlist->owner_type    = 'user';
      $playlist->owner_id      = Engine_Api::_()->user()->getViewer()->getIdentity();
      $playlist->description   = trim($values['description']);
      $playlist->search        = $values['search'];
      $playlist->save();
      $values['playlist_id']   = $playlist->playlist_id;

      // Assign $playlist to a Core_Model_Item
      $playlist = $this->_playlist = Engine_Api::_()->getItem('music_playlist', $values['playlist_id']);

      // get file_id list
      $file_ids = array();
      foreach (explode(' ', $values['fancyuploadfileids']) as $file_id) {
        $file_id = trim($file_id);
        if (!empty($file_id))
          $file_ids[] = $file_id;
      }

      // Attach songs (file_ids) to playlist
      if (!empty($file_ids))
        foreach ($file_ids as $file_id)
          $playlist->addSong($file_id);

      // Only create activity feed item if "search" is checked
      if ($playlist->search) {
        $activity = Engine_Api::_()->getDbtable('actions', 'activity');
        $action   = $activity->addActivity(
            Engine_Api::_()->user()->getViewer(),
            $playlist,
            'music_playlist_new',
            null,
            array('count' => count($file_ids))
        );
        if (null !== $action)
          $activity->attachActivity($action, $playlist);
      }
    }
      



    // Authorizations
    $auth      = Engine_Api::_()->authorization()->context;
    $prev_allow_comment = $prev_allow_view = false;
    foreach ($this->_roles as $role => $role_label) {
      // allow viewers
      if ($values['auth_view'] == $role || $prev_allow_view) {
        $auth->setAllowed($playlist, $role, 'view', true);
        $prev_allow_view = true;
      } else
        $auth->setAllowed($playlist, $role, 'view', 0);

      // allow comments
      if ($values['auth_comment'] == $role || $prev_allow_comment) {
        $auth->setAllowed($playlist, $role, 'comment', true);
        $prev_allow_comment = true;
      } else
        $auth->setAllowed($playlist, $role, 'comment', 0);
    }




    if (!empty($values['art']))
      $playlist->setPhoto($this->art);

    return $playlist;
  }

}
