<?php
/**
 * SocialEngine
 *
 * @category   Application_Extensions
 * @package    Music
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 * @version    $Id: AdminController.php 4379 2010-03-16 03:00:07Z jung $
 * @author     Stephen
 */

/**
 * @category   Application_Extensions
 * @package    Music
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 */
class Music_AdminLevelController extends Core_Controller_Action_Admin
{
  public function init()
  {
    $this->view->navigation = $navigation = Engine_Api::_()->getApi('menus', 'core')
      ->getNavigation('music_admin_main', array(), 'music_admin_main_level');
  }

  public function indexAction()
  {
    // Get level id
    if( null !== ($id = $this->_getParam('level_id')) ) {
      $level = Engine_Api::_()->getItem('authorization_level', $id);
    } else {
      $level = Engine_Api::_()->getItemTable('authorization_level')->getDefaultLevel();
    }

    if( !$level instanceof Authorization_Model_Level ) {
      throw new Engine_Exception('missing level');
    }

    $id = $level->level_id;

    // Make form
    $this->view->form = $form = new Music_Form_Admin_Level();
    $form->level_id->setValue($id);

    $permissionsTable = Engine_Api::_()->getDbtable('permissions', 'authorization');

    // Check post
    if( !$this->getRequest()->isPost() ) {
      $form->populate($permissionsTable->getAllowed('music_playlist', $id, array_keys($form->getValues())));
      return;
    }

    // Check validitiy
    if( !$form->isValid($this->getRequest()->getPost()) ) {
      return;
    }

    // Process

    $values = $form->getValues();

    $db = $permissionsTable->getAdapter();
    $db->beginTransaction();

    try
    {
      // Set permissions
      $permissionsTable->setAllowed('music_playlist', $id, $values);

      // Commit
      $db->commit();
    }

    catch( Exception $e )
    {
      $db->rollBack();
      throw $e;
    }

  }


  // legacy code
  /*
  public function oldindexAction()
  {
    $default_level_id  = Engine_Api::_()->getApi('settings', 'core')->authorization_defaultlevel;
    $settings          = array('level_id' => $this->getRequest()->getParam('level_id', $default_level_id));

    $permissionTable   = Engine_Api::_()->getDbtable('permissions', 'authorization');
    $select            = $permissionTable->select()
                                         ->where('level_id = ?', $settings['level_id'])
                                         ->where('type = ?', 'music_playlist');
    foreach ($permissionTable->fetchAll($select) as $row) {
      $settings[$row->name]   = $row->value;
      if (!empty($row->params)) {
        $settings[$row->name] = $row->params;
      }
    }

    $this->view->form = new Music_Form_Admin_Level();
    $this->view->form->populate($settings);
    if ($settings['level_id'] == Engine_Api::_()->getDbtable('levels', 'authorization')->getPublicLevel()->level_id) {
      $this->view->form->removeElement('moderator');
    }

    // process form
    if ( $this->getRequest()->isPost() && $this->view->form->isValid($this->getRequest()->getPost()) ) {
      $db = Engine_Api::_()->getDbTable('playlists', 'music')->getAdapter();
      $db->beginTransaction();
      try {
        foreach ($this->view->form->getElements() as $element) {
          $name = $element->getName();
          if ($name == 'level_id' || $name == 'submit')
            continue;
          $settings[$name] = $this->getRequest()->getParam($name);
          $element_select  = clone $select;
          $element_select->where('name = ?', $name);
          $row = $permissionTable->fetchRow($element_select);
          if (!($row instanceof Engine_Db_Table_Row))
            $row = $permissionTable->createRow(array(
                      'level_id' => $settings['level_id'],
                      'type' => 'music_playlist',
                      'name' => $name,
                    ));
          if (is_array($element->getValue())) {
            $row->value  = 1;
            $row->params = Zend_Json_Encoder::encode($element->getValue());
          } elseif (is_numeric($element->getValue()) && $element->getValue() < 3) {
            $row->value  = $element->getValue();
            $row->params = null;
          } else {
            $row->value    = 1;
            $row->params   = $element->getValue();
          }
          $row->save();
        }
        // overrides for moderators
        if ($this->getRequest()->getParam('moderator')) {
          foreach (array('create', 'view', 'edit', 'delete', 'moderator') as $role) {
            $element_select  = clone $select;
            $element_select->where('name = ?', $role);
            $row = $permissionTable->fetchRow($element_select);
            $row->value = 2;
            $row->save();
          }
        }
        $this->view->form->populate($settings);
        $db->commit();
      } catch (Exception $e) {
        $db->rollback();
        throw $e;
      }
    }
  }*/
}