<?php
/**
 * SocialEngine
 *
 * @category   Application_Core
 * @package    User
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 * @version    $Id: Login.php 5184 2010-04-22 04:14:11Z john $
 */

/**
 * @category   Application_Core
 * @package    User
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 */
class User_Form_Login extends Engine_Form
{
  public function init()
  {
    // Init form
    $this->setTitle('Member Sign In');
    $this->setDescription('If you already have an account, please enter your '.
      'details below. If you don\'t have one yet, please <a href="'.
      Zend_Controller_Front::getInstance()->getRouter()->assemble(array(), 'user_signup', true).
      '">sign up</a> first.');
    $this->setAttrib('id', 'user_form_login');
    $this->loadDefaultDecorators();
    $this->getDecorator('Description')->setOption('escape', false);

    // Init email
    $this->addElement('Text', 'email', array(
      'label' => 'Email Address',
      'required' => true,
      'allowEmpty' => false,
      'validators' => array(
        'EmailAddress'
      ),
      'tabindex' => 1,
    ));

    // Init password
    $this->addElement('Password', 'password', array(
      'label' => 'Password',
      'required' => true,
      'allowEmpty' => false,
      'tabindex' => 2,
    ));
    
    // Init remember me
    $this->addElement('Checkbox', 'remember', array(
      'label' => 'Remember Me',
      'tabindex' => 3,
    ));

    $this->addElement('Hidden', 'return_url', array(
      
    ));

    if (Engine_Api::_()->getApi('settings', 'core')->core_spam_login) {
      $this->addElement('captcha', 'captcha', array(
        'label' => 'Human Verification',
        'description' => 'Please validate that you are not a robot by typing in the letters and numbers in this image:',
        'captcha' => 'image',
        'captchaOptions' => array(
          'wordLen' => 6,
          'fontSize' => '30',
          'timeout' => 300,
          'imgDir' => APPLICATION_PATH . '/public/temporary/',
          'imgUrl' => Zend_Controller_Front::getInstance()->getBaseUrl().'/public/temporary/',
          'font' => APPLICATION_PATH . '/application/modules/Core/externals/fonts/arial.ttf'
        )));
    }

    // Init submit
    $this->addElement('Button', 'submit', array(
      'label' => 'Sign In',
      'type' => 'submit',
      'ignore' => true,
      'tabindex' => 4,
    ));

    $this->addDisplayGroup(array(
      'submit',
      'remember'
    ), 'buttons');

    // Init forgot password link
    $this->addElement('Dummy', 'forgot', array(
      'content' => '<span><a href="'.Zend_Controller_Front::getInstance()->getRouter()->assemble(array('module' => 'user', 'controller' => 'auth', 'action' => 'forgot'), 'default', true).'">Forgot Password?</a></span>',
    ));

    // Set default action
    $this->setAction(Zend_Controller_Front::getInstance()->getRouter()->assemble(array(), 'user_login'));
  }
}