<?php
/**
 * SocialEngine
 *
 * @category   Application_Core
 * @package    User
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 * @version    $Id: Account.php 5120 2010-04-17 03:16:14Z john $
 */

/**
 * @category   Application_Core
 * @package    User
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 */
class User_Plugin_Signup_Account extends Core_Plugin_FormSequence_Abstract
{
  protected $_name = 'account';

  //protected $_title = 'Create Account';

  protected $_formClass = 'User_Form_Signup_Account';

  protected $_script = array('signup/form/account.tpl', 'user');

  protected $_adminFormClass = 'User_Form_Admin_Signup_Account';

  protected $_adminScript = array('admin-signup/account.tpl', 'user');

  public $email = null;

  public function onProcess()
  {
    $settings = Engine_Api::_()->getApi('settings', 'core');
    $random =  ($settings->getSetting('user.signup.random', 0) == 1);
    $data = $this->getSession()->data;
    if ($random)
    {
      $data['password'] = Engine_Api::_()->user()->randomPass(10);
    }
    $user = Engine_Api::_()->getDbtable('users', 'user')->createRow();
    $user->setFromArray($data);
    $user->save();
    Engine_Api::_()->user()->setViewer($user);

    // Increment signup counter
    Engine_Api::_()->getDbtable('statistics', 'core')->increment('user.creations');


    if ($user->verified && $user->enabled) 
    {
      // Create activity for them
      Engine_Api::_()->getDbtable('actions', 'activity')->addActivity($user, $user, 'signup');
      // Set user as logged in if not have to verify email
      Engine_Api::_()->user()->getAuth()->getStorage()->write($user->getIdentity());
    }

    $email_params = array(
      'displayname' => $user->getTitle(),
      'email' => $user->email,
      'link' => 'http://' . $_SERVER['HTTP_HOST'] . Zend_Controller_Front::getInstance()->getRouter()->assemble(array(), 'user_login'),
    );
    if ($random)
    {
      $email_params['password'] = $data['password'];
    }    
    if (($settings->getSetting('user.signup.verifyemail', 0)) == 1)
    {
      Engine_Api::_()->getApi('mail', 'core')->sendSystem(
        $user,
        $random ? 'core_welcome_password':'core_welcome',
        $email_params
      );
    }
    if (($settings->getSetting('user.signup.verifyemail', 1)) == 2)
    {
      $verify_table = Engine_Api::_()->getDbtable('verify', 'user');
      $verify_row = $verify_table->createRow();
      $verify_row->user_id = $user->getIdentity(); 
      $verify_row->code = md5($user->email . $user->creation_date . $settings->getSetting('core.secret', 'staticSalt') . (string) rand(1000000, 9999999));
      $verify_row->date = $user->creation_date;
      $verify_row->save();
      $email_params['link'] =  'http://' . $_SERVER['HTTP_HOST'] . Zend_Controller_Front::getInstance()->getRouter()->assemble(array('action'=>'verify', 'email'=>$user->email, 'verify'=>$verify_row->code), 'user_signup');
      Engine_Api::_()->getApi('mail', 'core')->sendSystem(
        $user,
        $random ? 'core_verification_password' : 'core_verification',
        $email_params
      );
      }
  }

  public function onAdminProcess($form)
  {
    $settings = Engine_Api::_()->getApi('settings', 'core');
    $values = $form->getValues();
    $settings->user_signup = $values;
    if ($values['inviteonly'] == 1)
    {
      $step_table = Engine_Api::_()->getDbtable('signup', 'user');
      $step_row = $step_table->fetchRow($step_table->select()->where('class = ?', 'User_Plugin_Signup_Invite'));
      $step_row->enable = 0;
    }
  }

}