<?php
/**
 * SocialEngine
 *
 * @category   Application_Core
 * @package    User
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 * @version    $Id: IndexController.php 5606 2010-05-11 06:29:30Z john $
 */

/**
 * @category   Application_Core
 * @package    User
 * @copyright  Copyright 2006-2010 Webligo Developments
 * @license    http://www.socialengine.net/license/
 */
class User_IndexController extends Core_Controller_Action_Standard
{
  public function indexAction()
  {

  }

  public function browseAction()
  {
    $this->view->form = $form = new User_Form_Search(array(
      'type' => 'user'
    ));

    $lastrow = $this->_getParam('lastrow', 0);
    $ajax = $this->_getParam('ajax', false);
    $totalUsers = $this->_getParam('totalUsers', false);


    $table = $this->_helper->api()->getDbtable('users', 'user');
    $select = $table->select()
      ->where('search = ?', 1)
      ->limit(10, $lastrow)
      ->order('displayname');
    
    if(!$ajax) $totalUsers = count($table->fetchAll($table->select()->where('search = ?', 1)));

    $this->view->lastrow = $lastrow + 10;
    $this->view->users = $table->fetchAll($select);
    $this->view->totalUsers = $totalUsers;
    $this->view->ajax = $ajax;
    $this->view->userCount = count($this->view->users);
  }

  public function homeAction()
  {
    if( !Engine_Api::_()->user()->getViewer()->getIdentity() )
    {
      return $this->_helper->redirector->gotoRoute(array(), 'home', true);
    }
    $this->_helper->content->render();
    //$this->_helper->content->setNoRender()->render();
  }

  public function searchAction()
  {
    $this->view->form = $form = new Fields_Form_Search(array(
      'type' => 'user'
    ));
    
    // Get search params
    $params = $this->getRequest()->getPost();
    $searchParts = Engine_Api::_()->fields()->getSearchQuery('user', $params);

    // Get other params
    $defaults = array(
      'form_lastrow' => 0,
      'form_ajax' => false,
      'totalUsers' => false,

      'is_online' => null,
      'has_photo' => null,
      'displayname' => null,
    );
    $options = array_merge($defaults, array_intersect_key($this->_getAllParams(), $defaults));

    extract($options); // form_lastrow, form_ajax, totalUsers, is_online, has_photo

    $ajax = $form_ajax; // tmp b/c
    $lastrow = $form_lastrow; //tmp b/c

    // Get table info
    $table = $this->_helper->api()->getDbtable('users', 'user');
    $userTableName = $table->info('name');
    $searchTable = Engine_Api::_()->fields()->getTable('user', 'search');
    $searchTableName = $searchTable->info('name');

    // Contruct query
    $select = $table->select()
      //->setIntegrityCheck(false)
      ->from($userTableName)
      ->joinLeft($searchTableName, "`{$searchTableName}`.`item_id` = `{$userTableName}`.`user_id`", null)
      //->group("{$userTableName}.user_id")
      ->where("{$userTableName}.search = ?", 1)
      ->order("{$userTableName}.displayname ASC");
    
    // Build the photo and is online part of query
    if( !empty($has_photo) ) {
      $select->where($userTableName.'.photo_id != ?', "0");
    }
    if( !empty($is_online) ) {
      $select
        ->joinRight("engine4_user_online", "engine4_user_online.user_id = `{$userTableName}`.user_id", null)
        ->group("engine4_user_online.user_id")
        ->where($userTableName.'.user_id != ?', "0");
    }

    // Add displayname
    if( !empty($displayname) ) {
      $select->where("`{$userTableName}`.`displayname` LIKE ?", "%{$displayname}%");
    }
    
    // Build search part of query
    foreach( $searchParts as $k => $v ) {
      $select->where("`{$searchTableName}`.{$k}", $v);
    }

    // Get total
    if( !$ajax ) {
      // Hopefully this won't cause it to select them
      $tmpPaginator = Zend_Paginator::factory($select);
      $totalUsers = $tmpPaginator->getTotalItemCount();
    }

    // Set limits
    $select->limit(10, $lastrow);
    
    // Get data
    $users = $table->fetchAll($select);

    // Debug
    //echo $select->__toString();

    $this->view->lastrow = $lastrow + 10;
    $this->view->users = $users;
    $this->view->totalUsers = $totalUsers;
    $this->view->ajax = $ajax;
    $this->view->userCount = count($this->view->users);
  }
}